<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine\Content
 */

namespace DeubertIt\ElasticApmDoctrine\Context;

use DeubertIt\ElasticApmDoctrine\Listener\ElasticApmDoctrineListener;

/**
 * Class Context
 * @package DeubertIt\ElasticApmDoctrine\Content
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class Context
{
    /**
     * @var array
     */
    protected $profilers;

    /**
     * @return array
     */
    public function getProfilers()
    {
        return $this->profilers;
    }

    /**
     * @param array $profilers
     * @return $this
     */
    public function setProfilers(array $profilers = [])
    {
        $this->profilers = $profilers;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getSummary()
    {
        if (!$this->getProfilers()) {
            return null;
        }

        $data = [];

        foreach ($this->getProfilers() as $profiler) {
            $queries = 0;
            $time = 0;
            /** @var \Doctrine_Event $event */
            foreach ($profiler as $event) {
                if (in_array($event->getCode(), ElasticApmDoctrineListener::getQueryEventCodes())) {
                    $time += $event->getElapsedSecs();
                    $queries += 1;
                }
            }
            $data[] = $queries . ' in ' . round($time * 1000, 2)  . ' ms';
        }
        $summary = implode(' / ', $data);

        return $summary;
    }

    /**
     * @return array|null
     */
    public function getData()
    {
        if (!$this->getProfilers()) {
            return null;
        }

        $data = [];

        foreach ($this->getProfilers() as $name => $profiler) {
            $data[$name]['connectionName'] = $name;
            /** @var \Doctrine_Event $event */
            foreach ($profiler as $event) {
                if (in_array($event->getCode(), ElasticApmDoctrineListener::getQueryEventCodes())) {
                    $eventContext = new EventContext();
                    $eventContext->setEvent($event);
                    $data[$name][] = $eventContext->getContext();
                }
            }
        }

        return $data;
    }

    /**
     * @return array|null
     */
    public function getContext()
    {
        if (!$this->getProfilers()) {
            return null;
        }

        return [
            'summary' => $this->getSummary(),
            'data' => $this->getData(),
        ];
    }
}
