<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine
 */

namespace DeubertIt\ElasticApmDoctrine\Context;

/**
 * Class EventContext
 * @package DeubertIt\ElasticApmDoctrine\Context
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class EventContext
{
    /**
     * @var \Doctrine_Event|null
     */
    protected $event;

    /**
     * @return \Doctrine_Event|null
     */
    public function getEvent()
    {
        return $this->event;
    }

    /**
     * @param \Doctrine_Event|null $event
     * @return $this
     */
    public function setEvent($event)
    {
        $this->event = $event;
        return $this;
    }

    /**
     * @return array|null
     */
    public function getContext()
    {
        if (!$this->getEvent()) {
            return null;
        }

        $query = $this->getEvent()->getQuery();
        //$query = htmlspecialchars($this->getEvent()->getQuery());
        //$query = preg_replace('/\b(SELECT|AS|FROM|WHERE|ORDER BY|GROUP BY|LIMIT|ON|LEFT JOIN|JOIN)\b/','<strong>$1</strong>', $query);

        $time = '[' . round($this->getEvent()->getElapsedSecs() * 1000, 2) . ' ms]';

        $params = $this->getEvent()->getParams();
        if (!empty($params)) {
            $params = array_map('htmlspecialchars', $params);
        }

        return [
            'query' => $query,
            'time' => $time,
            'params' => $params
        ];
    }
}
