<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine
 */

namespace DeubertIt\ElasticApmDoctrine;

use DeubertIt\ElasticApm\Base;
use DeubertIt\ElasticApmDoctrine\Listener\ElasticApmDoctrineListener;
use DeubertIt\ElasticApmDoctrine\Transaction\Bag;
use PhilKra\Agent;
use PhilKra\Events\Transaction;
use DeubertIt\ElasticApmZfMvc\ElasticApmZfMvc;

/**
 * Class ElasticApmDoctrine
 * @package DeubertIt\ElasticApmDoctrine
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class ElasticApmDoctrine
{
    /**
     * @var bool
     */
    protected static $enabled = false;

    /**
     * @var Agent
     */
    protected static $agent;

    /**
     * @var int
     */
    public static $transactionNameLength = 100;

    /**
     * @return bool
     */
    public static function isEnabled()
    {
        return self::$enabled;
    }

    public static function enable()
    {
        self::$enabled = true;
        ElasticApmDoctrineListener::listen();
    }

    public static function disable()
    {
        self::$enabled = false;
    }

    /**
     * @return Agent
     * @throws \Exception
     */
    public static function getAgent()
    {
        if (is_null(self::$agent)) {
            self::$agent = Base::getRegistryInstance();
        }
        return self::$agent;
    }

    /**
     * @param Agent|null $agent
     */
    public static function setAgent(Agent $agent = null)
    {
        self::$agent = $agent;
    }

    /**
     * @param int $lenght
     */
    public static function setTransactionNameLenght(int $lenght = null)
    {
        self::$transactionNameLength = $lenght;
    }

    /**
     * @param string $name
     * @param Transaction|null $parent
     * @return Transaction|null
     * @throws \Exception
     */
    public static function start(string $name = null, Transaction $parent = null)
    {
        try {
            if (!Bag::getInstance()->transactionAlreadyExists($name)) {
                $name = Bag::getInstance()->getNewTransactionName($name);
            }

            $transaction = self::getAgent()->startTransaction($name);
            if (!is_null($parent)) {
                $transaction->setParent($parent);
            }
            Bag::getInstance()->startedTransaction($transaction);

            return $transaction;
        } catch (\Exception $e) {
            throw $e;
        }

        return null;
    }

    /**
     * @param Transaction|string $transaction
     * @throws \Exception
     */
    public static function stop($transaction)
    {
        try {
            $name = $transaction;
            if ($transaction instanceof Transaction) {
                $name = $transaction->getTransactionName();
            }

            if (Bag::getInstance()->transactionAlreadyExists($name) && !Bag::getInstance()->transactionIsStopped($name)) {
                self::getAgent()->stopTransaction($name);
                Bag::getInstance()->stoppedTransaction($name);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param string $name
     * @return Transaction|null
     */
    public static function getTransaction(string $name)
    {
        try {
            return self::getAgent()->getTransaction($name);
        } catch (\Exception $e) {}

        return null;
    }

    /**
     * @param Transaction|string|null $trasnaction
     * @param array|null $context
     * @return Transaction
     */
    public static function setTransactionContext($trasnaction, array $context = null)
    {
        if (is_string($trasnaction)) {
            $trasnaction = self::getTransaction($trasnaction);
        }

        if (!is_null($context)) {
            $trasnaction->setCustomContext($context);
        }

        return $trasnaction;
    }

    /**
     * @param string $query
     * @return string
     */
    public static function generateNameFromQuery(string $query)
    {
        $name = str_replace('`', '', $query);

        if (!is_null(self::$transactionNameLength) && self::$transactionNameLength > 0) {
            $ext = '';
            if (strlen($name) > self::$transactionNameLength) {
                $ext = ' ...';
            }

            $name = substr($name, 0, self::$transactionNameLength) . $ext;
        }

        return $name;
    }

    /**
     * Get parent transaction
     * If ElasticApmZfMvc::$transaction (current ElasticApmZfMvc transaction) exists, this will be the parent
     * Else main transaction will be the parent
     *
     * @return Transaction|null
     * @throws \Exception
     */
    public static function getParentTransaction()
    {
        $parentTransaction = null;

        if (class_exists(ElasticApmZfMvc::class)) {
            $isEnabled = ElasticApmZfMvc::isEnabled();
            if ($isEnabled) {
                $parentTransaction = ElasticApmZfMvc::getCurrentTransaction();
            } else {
                $parentTransaction = self::getMainTransaction();
            }
        } else {
            $parentTransaction = self::getMainTransaction();
        }

        return $parentTransaction;
    }

    /**
     * @return Transaction|null
     * @throws \Zend_Exception
     */
    protected static function getMainTransaction()
    {
        if (\Zend_Registry::isRegistered('elastic-apm-agent_maintransaction')) {
            return \Zend_Registry::get('elastic-apm-agent_maintransaction');
        }
        return null;
    }
}
