<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine
 */

namespace DeubertIt\ElasticApmDoctrine\Listener;

use DeubertIt\ElasticApmDoctrine\Context\Context;
use DeubertIt\ElasticApmDoctrine\Profiler\ElasticApmDoctrineProfiler;

/**
 * Class ElasticApmDoctrineListener
 * @package DeubertIt\ElasticApmDoctrine\Listener
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class ElasticApmDoctrineListener
{
    /**
     * @var bool
     */
    protected static $listen = false;

    /**
     * @var array
     */
    protected static $profilers = [];

    /**
     * @return bool
     */
    public static function isListening()
    {
        return self::$listen;
    }

    /**
     * @return array
     */
    public static function getProfilers()
    {
        return self::$profilers;
    }

    /**
     * @param array $options
     */
    public static function listen(array $options = [])
    {
        if (!self::isListening()) {
            if (!isset($options['manager']) || !count($options['manager'])) {
                if (\Doctrine_Manager::getInstance()) {
                    $options['manager'] = \Doctrine_Manager::getInstance();
                }
            }

            foreach ($options['manager']->getIterator() as $connection) {
                self::$profilers[$connection->getName()] = new ElasticApmDoctrineProfiler();
                $connection->addListener(self::$profilers[$connection->getName()]);
            }
        }

        self::$listen = true;
    }

    public static function getContext()
    {
        $context = new Context();
        $context->setProfilers(self::$profilers);
        return $context->getContext();
    }

    /**
     * return codes for 'query' type of event
     */
    public static function getQueryEventCodes()
    {
        return array(
            \Doctrine_Event::CONN_EXEC,
            \Doctrine_Event::STMT_EXECUTE,
            \Doctrine_Event::CONN_QUERY,
        );
    }
}
