<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine
 */

namespace DeubertIt\ElasticApmDoctrine\Profiler;

use DeubertIt\ElasticApmDoctrine\Context\EventContext;
use DeubertIt\ElasticApmDoctrine\ElasticApmDoctrine;
use DeubertIt\ElasticApmDoctrine\Listener\ElasticApmDoctrineListener;
use PhilKra\Events\Transaction;

/**
 * Class ElasticApmDoctrineProfiler
 * @package DeubertIt\ElasticApmDoctrine\Profiler
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class ElasticApmDoctrineProfiler extends \Doctrine_Connection_Profiler
{
    /**
     * @var Transaction|null
     */
    protected $transaction;

    public function __call($m, $a)
    {
        // Start transaction on pre event
        if (substr($m, 0, 3) === 'pre') {
            /** @var \Doctrine_Event $event */
            $event = $a[0];
            if ($event instanceof \Doctrine_Event && in_array($event->getCode(), ElasticApmDoctrineListener::getQueryEventCodes())) {
                $name = ElasticApmDoctrine::generateNameFromQuery($event->getQuery());
                $transaction = ElasticApmDoctrine::start($name, ElasticApmDoctrine::getParentTransaction());
                $this->transaction = $transaction;
            }
        }

        parent::__call($m, $a);

        // Stop transaction on post event
        if (substr($m, 0, 3) !== 'pre') {
            /** @var \Doctrine_Event $event */
            $event = $a[0];
            if ($event instanceof \Doctrine_Event && in_array($event->getCode(), ElasticApmDoctrineListener::getQueryEventCodes())) {
                $eventContext = new EventContext();
                $eventContext->setEvent($event);

                $customContext = $eventContext->getContext();
                if (!is_null($customContext)) {
                    ElasticApmDoctrine::setTransactionContext($this->transaction, $customContext);
                }
                ElasticApmDoctrine::stop($this->transaction);
            }
        }
    }


}
