<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine
 */

namespace DeubertIt\ElasticApmDoctrine\Tests\Data;

use DeubertIt\ElasticApmDoctrine\Context\EventContext;
use DeubertIt\ElasticApmDoctrine\Tests\TestCase;

/**
 * Class EventContext
 * @package DeubertIt\ElasticApmDoctrine\Tests\Context
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class EventContextTests extends TestCase
{
    public function testContextData()
    {
        $doctrineEventMock = $this->getMockBuilder(\Doctrine_Event::class)
            ->disableOriginalConstructor()
            ->getMock();
        $doctrineEventMock->method('getCode')->willReturn(\Doctrine_Event::STMT_EXECUTE);
        $doctrineEventMock->method('getElapsedSecs')->willReturn(0.00031304359436035);
        $doctrineEventMock->method('getQuery')->willReturn('SELECT * FROM TABLE_NAME');
        $doctrineEventMock->method('getParams')->willReturn(['param1' => 'value1']);

        $profilers = [
            'default' => [$doctrineEventMock]
        ];

        $eventContext = new EventContext();
        $eventContext->setEvent($doctrineEventMock);
        $context = $eventContext->getContext();

        $expectedContext = [
            'query' => 'SELECT * FROM TABLE_NAME',
            'time' => '[0.31 ms]',
            'params' => [
                'param1' => 'value1',
            ],
        ];
        $this->assertEquals($expectedContext, $context);
    }
}
