<?php
/**
 * DeubertIt
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2020 All rights reserved
 * @package DeubertIt\ElasticApmDoctrine
 */

namespace DeubertIt\ElasticApmDoctrine\Transaction;

use PhilKra\Events\Transaction;

/**
 * Class Bag
 * @package DeubertIt\ElasticApmDoctrine\Transaction
 *
 * @author Ivan Petrov <ivan.petrov@deubert.it>
 */
class Bag
{
    const STATUS_STARTED = 1;
    const STATUS_STOPPED = 0;

    /**
     * @var $this
     */
    protected static $instance;

    /**
     * @var array|Transaction[]
     */
    protected $transations = [];

    protected function __construct() {}

    public static function getInstance()
    {
        if (is_null(self::$instance)) {
            self::$instance = new Bag();
        }
        return self::$instance;
    }

    /**
     * @return array|Transaction[]
     */
    public function getTransations()
    {
        return $this->transations;
    }

    /**
     * @param array|Transaction[] $transations
     * @return Bag
     */
    public function setTransations($transations)
    {
        $this->transations = $transations;
        return $this;
    }

    /**
     * @param string|Transaction $transaction transaction name or instance
     * @return $this
     */
    public function startedTransaction($transaction)
    {
        $name = $this->getTransactionName($transaction);
        $this->transations[$name] = self::STATUS_STARTED;
        return $this;
    }

    /**
     * @param string|Transaction $transaction transaction name or instance
     * @return $this
     */
    public function stoppedTransaction($transaction)
    {
        $name = $this->getTransactionName($transaction);
        if (isset($this->transations[$name])) {
            $this->transations[$name] = self::STATUS_STOPPED;
        }

        return $this;
    }

    /**
     * @param string|Transaction $transaction transaction name or instance
     * @return bool
     */
    public function transactionAlreadyExists($transaction)
    {
        $name = $this->getTransactionName($transaction);
        if (isset($this->transations[$name])) {
            return true;
        }
        return false;
    }

    /**
     * @param string|Transaction $transaction transaction name or instance
     * @return bool
     */
    public function transactionIsStopped($transaction)
    {
        $name = $this->getTransactionName($transaction);
        if (!isset($this->transations[$name]) || $this->transations[$name] == self::STATUS_STOPPED) {
            return true;
        }
        return false;
    }

    /**
     * @param string|Transaction $transaction transaction name or instance
     * @return string
     */
    protected function getTransactionName($transaction)
    {
        $name = $transaction;
        if ($transaction instanceof Transaction) {
            $name = $transaction->getTransactionName();
        }
        return $name;
    }

    /**
     * @param string $name
     * @return string
     */
    public function getNewTransactionName($name)
    {
        $count = 1;
        array_map(function ($key) use (&$count, $name) {
            if (strpos($key, $name) !== false) {
                $count++;
            }
        }, array_keys($this->transations));

        return $name.'_'.$count;
    }
}
